@@ -1,10 +1,11 @@
 import ast
 import asyncio
 import dataclasses
+import json
 import logging
 import re
 from copy import deepcopy
-from typing import List, Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import pandas as pd
 import pyjq
@@ -27,7 +28,7 @@ RE_JINJA_ALONE_IN_STRING = [RE_JINJA + r'([ )])', RE_JINJA + r'()$']
 
 RE_SET_KEEP_TYPE = r'{{__keep_type__\1}}\2'
 RE_GET_KEEP_TYPE = r'{{(__keep_type__[^({{)}]*)}}'
-RE_NAMED_PARAM = r'%\([a-zA-Z0-9_]*\)s'
+RE_NAMED_PARAM = r'\'?%\([a-zA-Z0-9_]*\)s\'?'
 
 
 class NonValidVariable(Exception):
@@ -286,7 +287,17 @@ class ConnectorStatus:
         return dataclasses.asdict(self)
 
 
-def convert_to_qmark_paramstyle(query_string: str, params_values: dict) -> str:
+def get_param_name(printf_style_argument: str) -> str:
+    # %(foobar)s -> foobar
+    # '%(foobar)s' -> foobar
+    if printf_style_argument.startswith("'"):
+        return printf_style_argument[3:-3]
+    return printf_style_argument[2:-2]
+
+
+def convert_to_qmark_paramstyle(
+    query_string: str, params_values: dict
+) -> Tuple[str, List[Optional[Any]]]:
     """Takes a query in pyformat paramstyle and transforms it in qmark
        by replacing placeholders by ? and returning values in right order
     ex :
@@ -295,7 +306,7 @@ def convert_to_qmark_paramstyle(query_string: str, params_values: dict) -> str:
         ('select * from test where id > ? and price > ?;', [1, 10])"""
 
     extracted_params = re.findall(RE_NAMED_PARAM, query_string)
-    qparams = [m[2:-2] for m in extracted_params]
+    qparams = [get_param_name(m) for m in extracted_params]
     ordered_values = [params_values.get(p) for p in qparams]
 
     # Check if we need to replace a list
@@ -375,3 +386,20 @@ def pandas_read_sql(
             raise
 
     return df
+
+
+def format_db_model(
+    unformatted_db_tree: List[Tuple[str, str, str, str, Dict[str, str]]]
+) -> List[Dict[str, Union[str, List[Dict[str, str]]]]]:
+    df = pd.DataFrame(unformatted_db_tree)
+    df.columns = ['database', 'schema', 'type', 'name', 'columns']
+    try:  # if columns is a string
+        df['columns'] = df['columns'].apply(json.loads)
+    except TypeError:  # else ignore
+        pass
+    return (
+        df.groupby(by=['schema', 'database', 'type', 'name'])['columns']
+        .apply(sum)
+        .reset_index()
+        .to_dict('records')
+    )
