@@ -288,7 +288,7 @@ class MongoConnector(ToucanConnector):
         )
         return _format_explain_result(result)
 
-    def get_unique_identifier(self) -> dict:
+    def get_unique_identifier(self) -> str:
         return self.json(
             exclude={'client'}
         )  # client is a MongoClient instance, not json serializable
