@@ -10,9 +10,15 @@ import redshift_connector
 from pydantic import Field, SecretStr, create_model, root_validator
 from pydantic.types import constr
 
-from toucan_connectors.common import ConnectorStatus
+from toucan_connectors.common import ConnectorStatus, format_db_model
 from toucan_connectors.connection_manager import ConnectionManager
-from toucan_connectors.redshift.data_types import types_map
+from toucan_connectors.redshift.utils import (
+    aggregate_columns,
+    build_database_model_extraction_query,
+    create_columns_query,
+    merge_columns_and_tables,
+    types_map,
+)
 from toucan_connectors.sql_query_helper import SqlQueryHelper
 from toucan_connectors.toucan_connector import (
     DataSlice,
@@ -75,12 +81,18 @@ class RedshiftDataSource(ToucanDataSource):
         'the table parameter',
         widget='sql',
     )
+    query_object: Dict = Field(
+        None,
+        description='An object describing a simple select query, this field is used internally',
+        **{'ui.hidden': True},
+    )
     table: constr(min_length=1) = Field(
         None,
         description='The name of the data table that you want to '
         'get (equivalent to "SELECT * FROM '
         'your_table")',
     )
+    language: str = Field('sql', **{'ui.hidden': True})
 
     def __init__(self, **data):
         super().__init__(**data)
@@ -99,7 +111,7 @@ class RedshiftDataSource(ToucanDataSource):
                 ds = RedshiftDataSource(
                     domain='Redshift', name='redshift', database=current_config['database']
                 )
-                available_tables = connector._retrieve_tables(datasource=ds)
+                available_tables = connector._retrieve_tables(database=ds.database)
                 constraints['table'] = strlist_to_enum('table', available_tables, None)
         return create_model('FormSchema', **constraints, __base__=cls).schema()
 
@@ -199,13 +211,13 @@ class RedshiftConnector(ToucanConnector):
             con_params['region'] = self.region
         return {k: v for k, v in con_params.items() if v is not None}
 
-    def _get_connection(self, datasource) -> redshift_connector.Connection:
+    def _get_connection(self, database) -> redshift_connector.Connection:
         """Establish a connection to an Amazon Redshift cluster."""
 
         def connect_function() -> redshift_connector.Connection:
             con = redshift_connector.connect(
                 **self._get_connection_params(
-                    database=datasource.database if datasource is not None else None,
+                    database=database if database else None,
                 ),
             )
             con.autocommit = True  # see https://stackoverflow.com/q/22019154
@@ -221,12 +233,12 @@ class RedshiftConnector(ToucanConnector):
             if not self._is_alive:
                 return connection.close()
 
-        connection: RedshiftConnector = redshift_connection_manager.get(
-            identifier=f'{self.get_identifier()}{datasource.database}{self.cluster_identifier}',
+        connection: redshift_connector.Connection = redshift_connection_manager.get(
+            identifier=f'{self.get_identifier()}{database}{self.cluster_identifier}',
             connect_method=connect_function,
             alive_method=alive_function,
             close_method=close_function,
-            save=True if datasource.database else False,
+            save=True if database else False,
         )
         if self.connect_timeout is not None:
             t = Thread(target=self.sleeper)
@@ -238,12 +250,12 @@ class RedshiftConnector(ToucanConnector):
         self._is_alive = False
 
     @contextmanager
-    def _get_cursor(self, datasource) -> redshift_connector.Cursor:
-        with self._get_connection(datasource=datasource) as conn, conn.cursor() as cursor:
+    def _get_cursor(self, database) -> redshift_connector.Cursor:
+        with self._get_connection(database=database) as conn, conn.cursor() as cursor:
             yield cursor
 
-    def _retrieve_tables(self, datasource) -> List[str]:
-        with self._get_cursor(datasource=datasource) as cursor:
+    def _retrieve_tables(self, database) -> List[str]:
+        with self._get_cursor(database=database) as cursor:
             cursor.execute(TABLE_QUERY)
             res = cursor.fetchall()
         return [table_name for (table_name,) in res]
@@ -263,7 +275,7 @@ class RedshiftConnector(ToucanConnector):
             prepared_query, prepared_query_parameters = SqlQueryHelper.prepare_limit_query(
                 datasource.query, datasource.parameters, offset, limit
             )
-        with self._get_cursor(datasource=datasource) as cursor:
+        with self._get_cursor(database=datasource.database) as cursor:
             cursor.execute(prepared_query, prepared_query_parameters)
             result: pd.DataFrame = cursor.fetch_dataframe()
             if result is None:
@@ -326,11 +338,24 @@ class RedshiftConnector(ToucanConnector):
             return ConnectorStatus(status=False, details=self._get_details(1, False), error=str(e))
         return ConnectorStatus(status=True, details=self._get_details(2, True), error=None)
 
-    def describe(self, data_source) -> Dict:
-        with self._get_cursor(datasource=data_source) as cursor:
+    def describe(self, data_source: RedshiftDataSource) -> Dict:
+        with self._get_cursor(database=data_source.database) as cursor:
             cursor.execute(DESCRIBE_QUERY.format(column=data_source.query.replace(';', '')))
             res = cursor.description
         return {
             col[0].decode('utf-8') if isinstance(col[0], bytes) else col[0]: types_map.get(col[1])
             for col in res
         }
+
+    def get_model(self, db_name: str):
+        with self._get_cursor(database=db_name) as cursor:
+            cursor.execute("""select datname from pg_database where datistemplate = false;""")
+            available_dbs = [db_name for (db_name,) in cursor.fetchall()]
+            databases_tree = []
+        for db in available_dbs:
+            with self._get_cursor(database=db_name) as cursor:
+                cursor.execute(create_columns_query(db))
+                cols = aggregate_columns(cursor.fetch_dataframe())
+                cursor.execute(build_database_model_extraction_query())
+                databases_tree += merge_columns_and_tables(cols, cursor.fetch_dataframe())
+        return format_db_model(databases_tree)
