@@ -2,7 +2,7 @@ import concurrent
 import json
 import logging
 from timeit import default_timer as timer
-from typing import Dict, List, Optional
+from typing import Any, Dict, List, Optional
 
 import pandas as pd
 from pydantic import Field, constr
@@ -54,6 +54,22 @@ class SfDataSource(ToucanDataSource):
         'This field is used internally',
         **{'ui.hidden': True},
     )
+    language: str = Field('sql', **{'ui.hidden': True})
+
+
+def build_database_model_extraction_query() -> str:
+    return """select t.table_catalog as database, t.table_schema as schema,
+    CASE WHEN t.table_type = 'BASE TABLE' THEN 'table' ELSE lower(t.table_type) END as type,
+    t.table_name as name,
+    ARRAY_AGG(object_construct('name', c.column_name, 'type', c.data_type)) as columns
+    from
+        information_schema.tables t
+    inner join information_schema.columns c on
+        t.table_name = c.table_name
+    where t.table_type in ('BASE TABLE', 'VIEW')
+    and t.table_schema not in  ('PG_CATALOG', 'INFORMATION_SCHEMA', 'PG_INTERNAL')
+    and t.table_name not in ('LOAD_HISTORY')
+    group by t.table_catalog, t.table_schema, t.table_name, t.table_type;"""
 
 
 class SnowflakeCommon:
@@ -251,14 +267,18 @@ class SnowflakeCommon:
             stats=stats,
         )
 
-    def get_warehouses(self, connection, warehouse_name: Optional[str] = None) -> List[str]:
+    def get_warehouses(
+        self, connection: SnowflakeConnection, warehouse_name: Optional[str] = None
+    ) -> List[str]:
         query = 'SHOW WAREHOUSES'
         if warehouse_name:
             query = f"{query} LIKE '{warehouse_name}'"
         res = self._execute_query(connection, query).to_dict().get('name')
         return [warehouse for warehouse in res.values()] if res else []
 
-    def get_databases(self, connection, database_name: Optional[str] = None) -> List[str]:
+    def get_databases(
+        self, connection: SnowflakeConnection, database_name: Optional[str] = None
+    ) -> List[str]:
         query = 'SHOW DATABASES'
         if database_name:
             query = f"{query} LIKE '{database_name}'"
@@ -296,3 +316,7 @@ class SnowflakeCommon:
         )
         res = {r.name: type_code_mapping.get(r.type_code) for r in describe_res}
         return res
+
+    def get_db_content(self, connection: SnowflakeConnection) -> List[Dict[str, Any]]:
+        query = build_database_model_extraction_query()
+        return self._execute_query(connection, query)
