@@ -1,3 +1,4 @@
+import concurrent
 import logging
 import uuid
 from contextlib import suppress
@@ -9,6 +10,7 @@ import snowflake
 from pydantic import Field, PrivateAttr, SecretStr, create_model
 from snowflake.connector import SnowflakeConnection
 
+from toucan_connectors.common import format_db_model
 from toucan_connectors.connection_manager import ConnectionManager
 from toucan_connectors.json_wrapper import JsonWrapper
 from toucan_connectors.oauth2_connector.oauth2connector import (
@@ -19,7 +21,10 @@ from toucan_connectors.snowflake.snowflake_connector import (
     AuthenticationMethod,
     SnowflakeDataSource,
 )
-from toucan_connectors.snowflake_common import SnowflakeCommon
+from toucan_connectors.snowflake_common import (
+    SnowflakeCommon,
+    build_database_model_extraction_query,
+)
 from toucan_connectors.toucan_connector import Category, DataSlice, ToucanConnector, strlist_to_enum
 
 logger = logging.getLogger(__name__)
@@ -279,3 +284,28 @@ class SnowflakeoAuth2Connector(ToucanConnector):
     @staticmethod
     def get_connection_manager():
         return connection_manager
+
+    def _get_connection_and_db_content(self, database: str, db_contents: List):
+        with self._get_connection(
+            database=database, warehouse=self.default_warehouse
+        ) as connection:
+            db_contents += SnowflakeCommon().get_db_content(connection).to_dict('records')
+
+    def get_model(self):
+        with self._get_connection() as connection:
+            databases = SnowflakeCommon().get_databases(connection=connection)
+        content_queries = []
+        for db in databases:
+            content_queries.append(build_database_model_extraction_query())
+        db_contents = []
+        with concurrent.futures.ThreadPoolExecutor() as executor:
+            futures = [
+                executor.submit(self._get_connection_and_db_content, db, db_contents)
+                for db in databases
+            ]
+            for future in concurrent.futures.as_completed(futures):
+                if future.exception():
+                    raise future.exception()
+                else:
+                    self.logger.info('query finished')
+        return format_db_model(db_contents)
