@@ -25,6 +25,8 @@ try:
 except ImportError:
     pass
 
+LOGGER = logging.getLogger(__name__)
+
 
 class DataStats(BaseModel):
     total_rows: Optional[int] = None
@@ -214,7 +216,7 @@ def get_oauth2_configuration(cls):
     """Return a tuple indicating if the connector is an oauth2 connector
     and in this case, where can the credentials be located
     """
-    oauth2_enabled = hasattr(cls, '_auth_flow') and getattr(cls, '_auth_flow') == 'oauth2'
+    oauth2_enabled = hasattr(cls, '_auth_flow') and 'oauth2' in getattr(cls, '_auth_flow')
     oauth2_credentials_location = None
     if hasattr(cls, '_oauth_trigger'):
         oauth2_credentials_location = getattr(cls, '_oauth_trigger')
@@ -415,7 +417,7 @@ class ToucanConnector(BaseModel, metaclass=ABCMeta):
         """
         return ConnectorStatus()
 
-    def get_unique_identifier(self) -> dict:
+    def get_unique_identifier(self) -> str:
         """
         Returns a serialized version of the connector's config.
         Override this method in connectors which have not-serializable properties.
@@ -447,14 +449,15 @@ class ToucanConnector(BaseModel, metaclass=ABCMeta):
         """
         unique_identifier = {
             'connector': self.get_unique_identifier(),
-            'permissions': permissions,
+            'permissions': nosql_apply_parameters_to_query(permissions, data_source.parameters)
+            if data_source
+            else permissions,
             'offset': offset,
             'limit': limit,
         }
 
         if data_source is not None:
             unique_identifier['datasource'] = self._render_datasource(data_source)
-
         json_uid = JsonWrapper.dumps(unique_identifier, sort_keys=True, default=hash)
         string_uid = str(uuid.uuid3(uuid.NAMESPACE_OID, json_uid))
         return string_uid
